package com.cback;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.logging.Logger;

import org.xnio.ChannelListener;
import org.xnio.IoUtils;
import org.xnio.Pool;
import org.xnio.Pooled;
import org.xnio.channels.StreamSourceChannel;

public abstract class DataReadListener implements ChannelListener<StreamSourceChannel> {

	private final static Logger LOG = Logger.getLogger(DataReadListener.class.getName());

	private final Pool<ByteBuffer> bufferPool;
	private final ByteArrayOutputStream byteArrayStream;

	public DataReadListener(final Pool<ByteBuffer> bufferPool) {
		this.bufferPool = bufferPool;
		this.byteArrayStream = new ByteArrayOutputStream();
	}

	public void setup(final StreamSourceChannel channel) {
		Pooled<ByteBuffer> resource = this.bufferPool.allocate();
		ByteBuffer buffer = resource.getResource();
		try {
			int r = 0;
			do {
				r = channel.read(buffer);
				if (r == 0) {
					channel.getReadSetter().set(this);
					channel.resumeReads();
				} else if (r == -1) {
					dataRead(this.byteArrayStream.toByteArray());
					IoUtils.safeClose(channel);
				} else {
					buffer.flip();
					while (buffer.hasRemaining()) {
						this.byteArrayStream.write(buffer.get());
					}
				}
			} while (r > 0);
		} catch (IOException e) {
			error(e);
		} finally {
			resource.free();
		}
	}
	
    @Override
    public void handleEvent(final StreamSourceChannel channel) {
        Pooled<ByteBuffer> resource = this.bufferPool.allocate();
        ByteBuffer buffer = resource.getResource();
        try {
            int r = 0;
            do {
                r = channel.read(buffer);
                if (r == 0) {
                    return;
                } else if (r == -1) {
					dataRead(this.byteArrayStream.toByteArray());
                    IoUtils.safeClose(channel);
                } else {
                    buffer.flip();
					while (buffer.hasRemaining()) {
						this.byteArrayStream.write(buffer.get());
					}
                }
            } while (r > 0);
        } catch (IOException e) {
            error(e);
        } finally {
            resource.free();
        }
    }

	protected abstract void dataRead(byte[] postData);

	protected abstract void error(IOException e);

}
